#!/usr/bin/env python
# -*- encoding: utf-8 -*-


class VFXPathBaseConfig:
    def __getitem__(self, item):
        return getattr(self, item)

    @classmethod
    def config_keys(cls) -> list:
        return [item for item in dir(cls) if not item.startswith("_") and item != "config_keys"]

    config_template: dict = {}

    # current run environ root path
    windows_root_path: str = ""
    mac_root_path: str = ""
    linux_root_path: str = ""

    # Custom Value resolves replace
    # If the environment variable does not exist
    # it will be replaced, which means that the priority of the environment variable is high
    custom_replaced_value: dict = {}


class Configuration(VFXPathBaseConfig):
    # The configuration path must be json
    file_path: str = ""
    current_target_path: str = ""
    current_work_path: str = ""




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
import os
import json
import logging

from vfxpaths.global_config import Configuration, VFXPathBaseConfig
from vfxpaths.ability.path_env_resolve import resolve_real_path

log = logging.getLogger("vfxpaths.initialization")


def init_pattern_config():
    """Initialise with config file"""
    config_path = os.environ.get("VFXPATHS_CONFIG_FILE")
    if config_path:
        Configuration.file_path = config_path
        load_config_file()
    else:
        find_global_config = globals()
        for item in find_global_config.keys():
            if item.startswith("VFX_paths_config"):
                maps_config_field(find_global_config[item])
                return


def maps_config_field(config_obj):
    for field in VFXPathBaseConfig.config_keys():
        current_value = getattr(config_obj, field, False)
        if current_value:
            setattr(Configuration, field, current_value)


def register_config_file(config_path: str = ""):
    if config_path == "":
        if Configuration.file_path == "":
            info: str = "config_path parameter cannot be empty"
            log.error(info)
            assert False, info
        else:
            load_config_file()
    else:
        Configuration.file_path = config_path
        load_config_file()


def register_config(pattern_data: dict):
    for field in VFXPathBaseConfig.config_keys():
        if pattern_data.get(field):
            setattr(Configuration, field, pattern_data.get(field))


def add_config_template(key: str, value: str):
    VFXPathBaseConfig.config_template[key] = value


def remove_config_template(key: str):
    if VFXPathBaseConfig.config_template.get(key):
        del VFXPathBaseConfig.config_template[key]


def cancel_all_config():
    pass


def cancel_config(field: str):
    """
    Clear the configuration of the specified field
    """
    pass


def load_config_file():
    config_data: dict = {}
    config_path = resolve_real_path(Configuration.file_path)
    if config_path.startswith("http"):
        # Get the configuration information through the network get method
        pass
    elif config_path.endswith(".json"):
        if not os.path.exists(config_path):
            log.error("config_path not exists")
            return
        with open(config_path, 'r') as json_file:
            config_data = json.load(json_file)
    else:
        log.error("config file read error")
        return

    register_config(config_data)


__all__ = ["init_pattern_config",
           "register_config_file",
           "register_config",
           "add_config_template",
           "remove_config_template",
           "cancel_all_config"]



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
import json
import os.path

from vfxpaths.global_config import Configuration
from vfxpaths.ability.path_env_resolve import resolve_real_path
from vfxpaths.ability.path_operation import check_file_exists, find_end_file, find_start_file, no_creation_exists
from vfxpaths.error import FileNotExists


class RW:
    work_path = Configuration.current_work_path
    target_path = Configuration.current_target_path

    def __init__(self, target_path: str = "", work_path: str = ""):
        if target_path != "":
            self.target_path = target_path

        if work_path:
            self.work_path = work_path

    @property
    def get_target_path(self) -> str:
        return resolve_real_path(self.target_path, None, self.work_path)

    def get_json_data(self, file_path: str = "") -> dict:
        if file_path == "":
            return self.__read_json_file(self.get_target_path)
        else:
            return self.__read_json_file(resolve_real_path(file_path, None, self.work_path))

    @no_creation_exists
    def write_json(self, data: dict, file_path: str = "", model: str = "w", auto_creation: int = 1):
        if isinstance(data, dict):
            raise TypeError("data parameter must be dict")
        if model == "w":
            self.__write_json_file(file_path, data)
        else:
            if os.path.exists(file_path):
                old_data: dict = self.__read_json_file(file_path)
                old_data.update(data)
                self.__write_json_file(file_path, old_data)

    def read_file(self, file_path: str = ""):
        if file_path == "":
            return self.__read_file(self.get_target_path)
        else:
            return self.__read_file(resolve_real_path(file_path, None, self.work_path))

    @no_creation_exists
    def write_file(self, content, file_path: str = "", model: str = "w", auto_creation: int = 1):
        if isinstance(content, list):
            self.__write_file("\n".join(content), file_path, model)
            return
        self.__write_file(content, file_path, model)

    @find_start_file
    def read_start_file(self, start_str: str, file_path: str = ""):
        return self.__read_file(file_path)

    @find_end_file
    def read_end_file(self, end_str: str, file_path: str = ""):
        return self.__read_file(file_path)

    @find_start_file
    def read_start_json(self, end_str: str, file_path: str = ""):
        return self.__read_json_file(file_path)

    @find_end_file
    def read_end_json(self, end_str: str, file_path: str = ""):
        return self.__read_json_file(file_path)

    @check_file_exists
    def __read_json_file(self, json_file: str) -> dict:
        if not json_file.endswith(".json"):
            return {}
        with open(json_file, 'r', encoding='utf-8') as f:
            data = json.load(f)
        return data

    @staticmethod
    def __write_json_file(json_file: str, data: dict):
        if not json_file.endswith(".json"):
            raise TypeError("The file must be json")
        with open(json_file, 'w', encoding="utf-8") as f:
            json.dump(data, f, indent=4, ensure_ascii=False)

    @check_file_exists
    def __read_file(self, file_path) -> str:
        with open(file_path, "r", encoding="utf-8") as f:
            text = f.read()
        return text

    @staticmethod
    def __write_file(content: str, file_path: str, model: str):
        with open(file_path, model, encoding="utf-8") as f:
            f.write(content)




#!/usr/bin/env python
# -*- encoding: utf-8 -*-

class VFXBaseError(Exception):
    """Vfx paths error"""


class AnalyseError(VFXBaseError):
    """Raised when the path cannot be resolved"""


class AnalyseEevError(VFXBaseError):
    """Error parsing environment variable to true path"""


class FormatError(VFXBaseError):
    """Raised when data cannot be formatted as a path"""


class NotFound(VFXBaseError):
    """Raise when an item cannot be found."""


class ResolveError(VFXBaseError):
    """Raise when a template reference can not be resolved."""


class FileNotExists(VFXBaseError):
    """The specified file does not exist"""




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
import logging
from copy import copy
import re
import functools
from collections import defaultdict

from vfxpaths.global_config import Configuration
from vfxpaths.error import NotFound, FormatError
from vfxpaths.ability.regex_match import MatchModel, RegexCompile
from vfxpaths.ability.path_env_resolve import resolve_real_path

log = logging.getLogger(__name__)


class BaseResolve:
    target_path = Configuration.current_target_path
    current_work_path = Configuration.current_work_path
    use_name = ""

    def __init__(self, target_path: str = "",
                 use_name: str = "",
                 current_work_path: str = "",
                 match_model=MatchModel.center
                 ):
        self.match_model = match_model

        if use_name != "":
            if Configuration.config_template.get(use_name):
                self.use_name = use_name
            else:
                msg: str = "use_name: arguments name not exists"
                log.error(msg)
                raise NotFound(msg)

        if target_path != "":
            self.target_path = target_path

        if current_work_path:
            self.current_work_path = resolve_real_path(current_work_path)

        self._get_target_path = resolve_real_path(self.target_path, None, self.current_work_path)
        self._get_dict_data: dict = {}
        self._current_match_target: str = ""
        self._all_match_target: dict = Configuration.config_template

    @property
    def get_target_path(self) -> str:
        if self.target_path == "":
            log.error("The target path is not set")
            return ""
        return self._get_target_path

    @staticmethod
    def copy_custom_fields():
        if Configuration.custom_replaced_value:
            new_custom_data = copy(Configuration.custom_replaced_value)
            return new_custom_data
        return

    @staticmethod
    def _replace_field(match, field_count):
        field = match.group('stand_in')
        field_count[field] += 1
        field += '{0:02d}'.format(field_count[field])
        expression = match.group('expression')
        if expression is None:
            expression = RegexCompile.str_path_split.value
        return r'(?P<{}>{})'.format(field, expression)


class Resolve(BaseResolve):
    def __init__(self, **key):
        super(Resolve, self).__init__(**key)
        self.__init_use_template()

    @property
    def get_match_target(self) -> str:
        return self._current_match_target

    @property
    def get_all_match_target(self) -> dict:
        return self._all_match_target

    @property
    def get_fields_only_list(self) -> list:
        if self._current_match_target == "":
            return []
        all_field = RegexCompile.field_match.value.findall(self._current_match_target)
        return list(set(all_field))

    @property
    def get_all_fields_only_list(self) -> list:
        if self._current_match_target == "":
            return []
        all_field = RegexCompile.field_match.value.findall(self._current_match_target)
        return all_field

    @property
    def get_value_only_list(self) -> list:
        if self._current_match_target == "":
            return []
        return list(self.get_dict_data.values())

    @property
    def get_dict_data(self) -> dict:
        if self._get_target_path == "":
            log.error("The target path is empty")
            return self._get_dict_data

        if self._current_match_target == "":
            for key, value in self._all_match_target:
                path_dict = self.__find_match_pattern(value)
                if path_dict:
                    return path_dict
        else:
            return self.__find_match_pattern(self._current_match_target)

    def dict_to_path(self, dict_data: dict) -> str:
        if not dict_data:
            return ""
        self.__init_use_template()

        if self._current_match_target == "":
            log.warning("Match_target is none")
            return ""
        full_path = RegexCompile.field_match.value.sub(functools.partial(self.__to_expression_value, data=dict_data),
                                                       self._current_match_target)
        if full_path:
            return full_path.replace("\\.", "/")
        return ""

    @staticmethod
    def __to_expression_value(func_match, data) -> str:
        placeholder = func_match.group(1)
        key_list = placeholder.split('.')

        for item in key_list:
            if not data.get(item):
                msg = 'format data error {0} Missing key parameters {1}.'.format(data, placeholder)
                log.error(msg)
                raise FormatError(msg)
            else:
                return data.get(item)

    def __init_use_template(self):
        if self.use_name != "":
            current_pattern = Configuration.config_template.get(self.use_name)
            if current_pattern:
                self._current_match_target = current_pattern.replace("\\", "/").replace(".", r"\.")
        else:
            log.warning("No template specified for use.")

    def __find_match_pattern(self, pattern: str) -> dict:
        match_expression = self.__convert_full_expression(pattern)
        result = match_expression.search(self._get_target_path)
        if not result:
            log.error("No mapping information was obtained")
            return {}
        result_data: dict = {}
        for key, value in result.groupdict().items():
            new_key: str = key[:-2]
            result_data[new_key] = value
        return result_data

    def __convert_full_expression(self, pattern: str):
        log.info("pattern: {}".format(pattern))
        expression = re.sub(r'{(?P<stand_in>.+?)(:(?P<expression>(\\}|.)+?))?}',
                            functools.partial(self._replace_field, field_count=defaultdict(int)),
                            pattern)
        expression = self.__select_model(expression)
        return re.compile(expression)

    def __select_model(self, expression):
        if self.match_model is MatchModel.stat:
            return "^{}".format(expression)
        elif self.match_model is MatchModel.end:
            return "{}$".format(expression)
        elif self.match_model is MatchModel.full:
            return "^{}$".format(expression)
        return expression



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
from vfxpaths.resolve import Resolve
from vfxpaths.utils import RW
from vfxpaths.ability.regex_match import MatchModel
from vfxpaths.initialization import (init_pattern_config,
                                     add_config_template,
                                     remove_config_template)


def init_config():
    # initialization config
    init_pattern_config()


__all__ = ["Resolve",
           "add_config_template",
           "remove_config_template",
           "MatchModel"]




#!/usr/bin/env python
# -*- encoding: utf-8 -*-
import os
import re
from setuptools import setup, find_packages


with open(os.path.join(os.path.dirname(__file__), 'vfxpaths', 'version.py')) as f:
    _version = re.match(r'.*__version__ = \'(.*?)\'', f.read(), re.DOTALL).group(1)


setup(
    name='vfxpaths',
    version=_version,
    description='Common processing module of VFX path',
    long_description=open('README.rst').read(),
    keywords='vfx, vfxpath, vfxfile, paths, read, write',
    url='https://github.com/VFXToolkits/vfxpaths',
    author='zuokangbo',
    author_email='zuokangbo@outlook.com',
    license='MIT License',
    packages=[
        'vfxpaths',
    ],
    python_requires='>=3.7',
    zip_safe=False
)
